#!/usr/bin/python

"""
A script to generate table of the classical measures of the tree class
(see tree.py) from a number of tree structures (written in MTG format).
"""

import sys
from tree import *

## Target QSM: Espoo maple
t0 = tree()
t0.read_mtg('Espoo.mtg',lpfg=False)

## The MTG file to process
mtg_file = 'out.mtg'
## Output file
out_file = 'measures.out'
## Mode for the output file
file_mode = "a"
## Process the input arguments
if len(sys.argv) >= 2:
	mtg_file = sys.argv[1]
	if len(sys.argv) >= 3:
		out_file = sys.argv[2]
		if len(sys.argv) >= 4:
			file_mode = sys.argv[3]

## Read in the tree structure to compare
t = tree()
t.read_mtg(mtg_file)

## Calculate the measures
h0 = t0.height()
g0 = t0.girth()
c0 = t0.crown_spread()
h = t.height()
g = t.girth()
c = t.crown_spread()

dh = abs(h0 - h)/h0
dg = abs(g0 - g)/g0
dc = abs(c0 - c)/c0

## Write to the file
fo = open(out_file, file_mode)
fo.write(repr(h) + ' ' + repr(g) + ' ' + repr(c) + ' ' + repr(dh) + ' ' + repr(dg) + ' ' + repr(dc) + "\n")

fo.close()
